@@ -5,5 +5,5 @@ ipr = lab.registry[0]
 if not isinstance(ipr, IPRoute):
     raise AssertionError('expected IPRoute instance')
 
-ipr.get_links.assert_called()
-ipr.close.assert_called_once()
+if not ipr.close.called:
+    print('\nWARNING: it is recommended to close IPRoute instances')
