@@ -123,7 +123,7 @@ class View(object):
 
     def _filter(self):
         ret = []
-        for (key, value) in tuple(self.src.items()):
+        for key, value in tuple(self.src.items()):
             try:
                 if self.constraint(key, value):
                     ret.append((key, value))
@@ -199,7 +199,9 @@ class Dotkeys(dict):
     __var_name = re.compile('^[a-zA-Z_]+[a-zA-Z_0-9]*$')
 
     def __dir__(self):
-        return [i for i in self if type(i) == str and self.__var_name.match(i)]
+        return [
+            i for i in self if isinstance(i, str) and self.__var_name.match(i)
+        ]
 
     def __getattribute__(self, key, *argv):
         try:
@@ -333,7 +335,7 @@ def hexload(data):
 
 def load_dump(f, meta=None):
     '''
-    Load a packet dump from an open file-like object.
+    Load a packet dump from an open file-like object or a string.
 
     Supported dump formats:
 
@@ -352,7 +354,14 @@ def load_dump(f, meta=None):
     code = None
     meta_data = None
     meta_label = None
-    for a in f.readlines():
+    if isinstance(f, str):
+        io_obj = io.StringIO()
+        io_obj.write(f)
+        io_obj.seek(0)
+    else:
+        io_obj = f
+
+    for a in io_obj.readlines():
         if code is not None:
             code += a
             continue
@@ -649,7 +658,7 @@ def metaclass(mc):
             slots = [slots]
         for k in slots:
             skip.append(k)
-        for (k, v) in cls.__dict__.items():
+        for k, v in cls.__dict__.items():
             if k not in skip:
                 nvars[k] = v
         return mc(cls.__name__, cls.__bases__, nvars)
