@@ -13,7 +13,7 @@ from pyroute2.ethtool.common import (
 )
 from pyroute2.ethtool.ioctl import WAKE_NAMES, IoctlEthtool
 from pyroute2.netlink.exceptions import NetlinkError
-from pyroute2.netlink.generic.ethtool import NlEthtool
+from pyroute2.netlink.generic.ethtool import NlEthtool, ethtool_rings_msg
 
 INT32MINUS_UINT32 = c_uint32(-1).value
 INT16MINUS_UINT16 = c_uint16(-1).value
@@ -71,6 +71,31 @@ class EthtoolFeatures(namedtuple('EthtoolFeatures', ('features',))):
             ioctl_features[feature.name] = feature.enable
 
 
+class EthtoolChannels(
+    namedtuple(
+        'EthtoolChannels',
+        (
+            "max_rx",
+            "max_tx",
+            "max_other",
+            "max_combined",
+            "rx_count",
+            "tx_count",
+            "other_count",
+            "combined_count",
+        ),
+    )
+):
+    @classmethod
+    def from_ioctl(cls, channels):
+        return cls(**{k: getattr(channels, k) for k in cls._fields})
+
+    @staticmethod
+    def to_ioctl(ioctl_channels, eth_channel):
+        for key, val in eth_channel.items():
+            setattr(ioctl_channels, key, val)
+
+
 class EthtoolWakeOnLan(namedtuple('EthtoolWolMode', ('modes', 'sopass'))):
     @classmethod
     def from_netlink(cls, nl_wol):
@@ -196,11 +221,9 @@ class EthtoolLinkMode(
 
     @classmethod
     def from_ioctl(cls, link_settings):
-        (
-            map_supported,
-            map_advertising,
-            map_lp_advertising,
-        ) = IoctlEthtool.get_link_mode_masks(link_settings)
+        (map_supported, map_advertising, map_lp_advertising) = (
+            IoctlEthtool.get_link_mode_masks(link_settings)
+        )
         bits_supported = IoctlEthtool.get_link_mode_bits(map_supported)
         supported_ports = []
         supported_modes = []
@@ -257,6 +280,108 @@ class EthtoolLinkMode(
         )
 
 
+class EthtoolRings(
+    namedtuple(
+        'EthtoolRings',
+        (
+            "rx_max",
+            "rx_mini_max",
+            "rx_jumbo_max",
+            "tx_max",
+            "rx",
+            "rx_mini",
+            "rx_jumbo",
+            "tx",
+            "rx_buf_len",
+            "tcp_data_split",
+            "cqe_size",
+            "tx_push",
+            "rx_push",
+            "tx_push_buf_len",
+            "tx_push_buf_len_max",
+        ),
+    )
+):
+    nl_attributs_dict = {
+        "rx_max": 'ETHTOOL_A_RINGS_RX_MAX',
+        "rx_mini_max": 'ETHTOOL_A_RINGS_RX_MINI_MAX',
+        "rx_jumbo_max": 'ETHTOOL_A_RINGS_RX_JUMBO_MAX',
+        "tx_max": 'ETHTOOL_A_RINGS_TX_MAX',
+        "rx": 'ETHTOOL_A_RINGS_RX',
+        "rx_mini": 'ETHTOOL_A_RINGS_RX_MINI',
+        "rx_jumbo": 'ETHTOOL_A_RINGS_RX_JUMBO',
+        "tx": 'ETHTOOL_A_RINGS_TX',
+        "rx_buf_len": 'ETHTOOL_A_RINGS_RX_BUF_LEN',
+        "tcp_data_split": 'ETHTOOL_A_RINGS_TCP_DATA_SPLIT',
+        "cqe_size": 'ETHTOOL_A_RINGS_CQE_SIZE',
+        "tx_push": 'ETHTOOL_A_RINGS_TX_PUSH',
+        "rx_push": 'ETHTOOL_A_RINGS_RX_PUSH',
+        "tx_push_buf_len": 'ETHTOOL_A_RINGS_TX_PUSH_BUF_LEN',
+        "tx_push_buf_len_max": 'ETHTOOL_A_RINGS_TX_PUSH_BUF_LEN_MAX',
+    }
+
+    def __new__(
+        cls,
+        rx_max=None,
+        rx_mini_max=None,
+        rx_jumbo_max=None,
+        tx_max=None,
+        rx=None,
+        rx_mini=None,
+        rx_jumbo=None,
+        tx=None,
+        rx_buf_len=None,
+        tcp_data_split=None,
+        cqe_size=None,
+        tx_push=None,
+        rx_push=None,
+        tx_push_buf_len=None,
+        tx_push_buf_len_max=None,
+    ):
+        return super(EthtoolRings, cls).__new__(
+            cls,
+            rx_max,
+            rx_mini_max,
+            rx_jumbo_max,
+            tx_max,
+            rx,
+            rx_mini,
+            rx_jumbo,
+            tx,
+            rx_buf_len,
+            tcp_data_split,
+            cqe_size,
+            tx_push,
+            rx_push,
+            tx_push_buf_len,
+            tx_push_buf_len_max,
+        )
+
+    @classmethod
+    def from_netlink(cls, nl_rings):
+        nl_rings = nl_rings[0]
+        return cls(
+            **{
+                cls_attr: nl_rings.get_attr(netlink_attr)
+                for cls_attr, netlink_attr in cls.nl_attributs_dict.items()
+            }
+        )
+
+    def to_netlink(self):
+        nl_rings_attrs = ethtool_rings_msg()
+        for cls_attr, netlink_attr in self.nl_attributs_dict.items():
+            attr = getattr(self, cls_attr)
+            if attr is not None:
+                nl_rings_attrs["attrs"].append((netlink_attr, attr))
+        return nl_rings_attrs
+
+    @classmethod
+    def from_ioctl(cls, ioctl_rings):
+        ioctl_rings = dict(ioctl_rings)
+        ioctl_rings.pop("cmd")
+        return cls(**ioctl_rings)
+
+
 class Ethtool:
     def __init__(self):
         self._with_ioctl = IoctlEthtool()
@@ -330,6 +455,31 @@ class Ethtool:
             wol_mode = self._with_ioctl.get_wol()
             return EthtoolWakeOnLan.from_ioctl(wol_mode)
 
+    def get_rings(self, ifname, with_netlink=None):
+        try:
+            rings = self._nl_exec(
+                self._with_nl.get_rings, with_netlink, ifname
+            )
+            rings = EthtoolRings.from_netlink(rings)
+        except UseIoctl:
+            self._with_ioctl.change_ifname(ifname)
+            rings_info = self._with_ioctl.get_rings()
+            rings = EthtoolRings.from_ioctl(rings_info)
+        return rings
+
+    def set_rings(self, ifname, with_netlink=None, **kwargs):
+        try:
+            rings = EthtoolRings(**kwargs).to_netlink()
+            self._nl_exec(self._with_nl.set_rings, with_netlink, rings, ifname)
+        except UseIoctl:
+            self._with_ioctl.change_ifname(ifname)
+            ioctl_rings = self._with_ioctl.get_rings()
+            for name, value in kwargs.items():
+                if name in ioctl_rings.keys() and ioctl_rings[name] != value:
+                    ioctl_rings[name] = value
+
+            self._with_ioctl.set_rings(ioctl_rings)
+
     def get_features(self, ifname):
         self._with_ioctl.change_ifname(ifname)
         return EthtoolFeatures.from_ioctl(self._with_ioctl.get_features())
@@ -340,6 +490,16 @@ class Ethtool:
         EthtoolFeatures.to_ioctl(ioctl_features, features)
         self._with_ioctl.set_features(ioctl_features)
 
+    def get_channels(self, ifname):
+        self._with_ioctl.change_ifname(ifname)
+        return EthtoolChannels.from_ioctl(self._with_ioctl.get_channels())
+
+    def set_channels(self, ifname, channels):
+        self._with_ioctl.change_ifname(ifname)
+        ioctl_channels = self._with_ioctl.get_channels()
+        EthtoolChannels.to_ioctl(ioctl_channels, channels)
+        self._with_ioctl.set_channels(ioctl_channels)
+
     def get_coalesce(self, ifname):
         self._with_ioctl.change_ifname(ifname)
         return EthtoolCoalesce.from_ioctl(self._with_ioctl.get_coalesce())
