@@ -506,6 +506,7 @@ class Interface(RTNL_Object):
         'alt_ifname_list': lambda x: list(json.loads(x or '[]'))
     }
     field_filter = LinkFieldFilter
+    old_ifname = None
 
     @classmethod
     def _count(cls, view):
@@ -654,6 +655,21 @@ class Interface(RTNL_Object):
         self._apply_script.append((do_add_vlan, {'spec': spec}))
         return self
 
+    @check_auth('obj:modify')
+    def ensure_vlan(self, spec):
+        def do_ensure_vlan(self, mode, spec):
+            try:
+                method = getattr(self.vlan.create(spec), mode)
+                return [method()]
+            except KeyError:
+                return []
+            except Exception as e_s:
+                e_s.trace = traceback.format_stack()
+                return [e_s]
+
+        self._apply_script.append((do_ensure_vlan, {'spec': spec}))
+        return self
+
     @check_auth('obj:modify')
     def del_vlan(self, spec):
         def do_del_vlan(self, mode, spec):
@@ -682,6 +698,23 @@ class Interface(RTNL_Object):
         self._apply_script.append((do_add_neighbour, {'spec': spec}))
         return self
 
+    @check_auth('obj:modify')
+    def ensure_neighbour(self, spec=None, **kwarg):
+        spec = spec or kwarg
+
+        def do_ensure_neighbour(self, mode, spec):
+            try:
+                method = getattr(self.neighbours.create(spec), mode)
+                return [method()]
+            except KeyError:
+                return []
+            except Exception as e_s:
+                e_s.trace = traceback.format_stack()
+                return [e_s]
+
+        self._apply_script.append((do_ensure_neighbour, {'spec': spec}))
+        return self
+
     @check_auth('obj:modify')
     def del_neighbour(self, spec=None, **kwarg):
         spec = spec or dict(kwarg)
@@ -727,6 +760,23 @@ class Interface(RTNL_Object):
         self._apply_script.append((do_add_ip, {'spec': spec}))
         return self
 
+    @check_auth('obj:modify')
+    def ensure_ip(self, spec=None, **kwarg):
+        spec = spec or kwarg
+
+        def do_ensure_ip(self, mode, spec):
+            try:
+                method = getattr(self.ipaddr.create(spec), mode)
+                return [method()]
+            except KeyError:
+                return []
+            except Exception as e_s:
+                e_s.trace = traceback.format_stack()
+                return [e_s]
+
+        self._apply_script.append((do_ensure_ip, {'spec': spec}))
+        return self
+
     @check_auth('obj:modify')
     def del_ip(self, spec=None, **kwarg):
         spec = spec or kwarg
@@ -798,17 +848,25 @@ class Interface(RTNL_Object):
         new_list = set(self['alt_ifname_list'])
         new_list.add(ifname)
         self['alt_ifname_list'] = list(new_list)
+        return self
 
     @check_auth('obj:modify')
     def del_altname(self, ifname):
         new_list = set(self['alt_ifname_list'])
         new_list.remove(ifname)
         self['alt_ifname_list'] = list(new_list)
+        return self
 
     @check_auth('obj:modify')
     def __setitem__(self, key, value):
         if key == 'peer':
             dict.__setitem__(self, key, value)
+        elif key == 'ifname':
+            if value in self['alt_ifname_list']:
+                self.del_altname(value)
+            if key in self and self.old_ifname is None:
+                self.old_ifname = self[key]
+            super(Interface, self).__setitem__(key, value)
         elif key == 'target' and self.state == 'invalid':
             dict.__setitem__(self, key, value)
         elif key == 'net_ns_fd' and self.state == 'invalid':
@@ -928,9 +986,19 @@ class Interface(RTNL_Object):
         return req
 
     @check_auth('obj:modify')
-    def apply_altnames(self, alt_ifname_setup):
-        alt_ifname_remove = set(self['alt_ifname_list']) - alt_ifname_setup
-        alt_ifname_add = alt_ifname_setup - set(self['alt_ifname_list'])
+    def apply_altnames(
+        self, alt_ifname_setup, alt_ifname_current, old_ifname=None
+    ):
+        if 'alt_ifname_list' in self.changed:
+            self.changed.remove('alt_ifname_list')
+        if alt_ifname_current is None:
+            # load the current state
+            self.load_from_system()
+            self.load_sql(set_state=False)
+            alt_ifname_current = set(self['alt_ifname_list'])
+
+        alt_ifname_remove = alt_ifname_current - alt_ifname_setup
+        alt_ifname_add = alt_ifname_setup - alt_ifname_current
         for ifname in alt_ifname_remove:
             self.sources[self['target']].api(
                 'link', 'property_del', index=self['index'], altname=ifname
@@ -939,8 +1007,11 @@ class Interface(RTNL_Object):
             self.sources[self['target']].api(
                 'link', 'property_add', index=self['index'], altname=ifname
             )
+        # reload alt ifnames from the system to check the state
         self.load_from_system()
         self.load_sql(set_state=False)
+        if old_ifname is not None and old_ifname in self['alt_ifname_list']:
+            alt_ifname_setup.add(old_ifname)
         if set(self['alt_ifname_list']) != alt_ifname_setup:
             raise Exception('could not setup alt ifnames')
 
@@ -953,9 +1024,14 @@ class Interface(RTNL_Object):
         setns = self.state.get() == 'setns'
         remove = self.state.get() == 'remove'
         alt_ifname_setup = set(self['alt_ifname_list'])
-        if 'alt_ifname_list' in self.changed:
-            self.changed.remove('alt_ifname_list')
+        old_ifname = self.old_ifname if 'ifname' in self.changed else None
         try:
+            if 'index' in self and (
+                self.old_ifname or 'alt_ifname_list' in self.changed
+            ):
+                self.apply_altnames(alt_ifname_setup, None)
+            if 'alt_ifname_list' in self.changed:
+                self.changed.remove('alt_ifname_list')
             super(Interface, self).apply(rollback, req_filter, mode)
             if setns:
                 self.load_value('target', self['net_ns_fd'])
@@ -964,7 +1040,9 @@ class Interface(RTNL_Object):
                 if spec:
                     self.state.set('system')
             if not remove:
-                self.apply_altnames(alt_ifname_setup)
+                self.apply_altnames(
+                    alt_ifname_setup, set(self['alt_ifname_list']), old_ifname
+                )
 
         except NetlinkError as e:
             if (
@@ -1010,6 +1088,8 @@ class Interface(RTNL_Object):
                 self.apply(rollback, req_filter, mode)
             else:
                 raise
+        finally:
+            self.old_ifname = None
         if ('net_ns_fd' in self.get('peer', {})) and (
             self['peer']['net_ns_fd'] in self.view.ndb.sources
         ):
@@ -1052,6 +1132,7 @@ class Interface(RTNL_Object):
                 self.ndb._event_queue.put(update)
 
     def load_from_system(self):
+        self.load_event.clear()
         (
             self.ndb._event_queue.put(
                 self.sources[self['target']].api(
@@ -1059,6 +1140,7 @@ class Interface(RTNL_Object):
                 )
             )
         )
+        self.load_event.wait(1)
 
     def load_sql(self, *argv, **kwarg):
         spec = super(Interface, self).load_sql(*argv, **kwarg)
