@@ -116,13 +116,12 @@ If you're using a PostgreSQL DB or a file based SQLite, simply drop
 all the tables from the DB, and NDB will create them from scratch
 on startup.
 '''
+
 import enum
-import inspect
 import json
 import random
 import sqlite3
 import sys
-import threading
 import time
 import traceback
 from collections import OrderedDict
@@ -130,15 +129,10 @@ from functools import partial
 
 from pyroute2 import config
 from pyroute2.common import basestring, uuid32
+from pyroute2.netlink import NLM_F_REPLACE
 
 #
-from .messages import cmsg
-from .objects import address, interface, neighbour, netns, route, rule
-
-try:
-    import queue
-except ImportError:
-    import Queue as queue
+from .objects import address, interface, neighbour, netns, probe, route, rule
 
 try:
     import psycopg2
@@ -148,7 +142,7 @@ except ImportError:
 #
 # the order is important
 #
-plugins = [interface, address, neighbour, route, netns, rule]
+plugins = [interface, address, neighbour, route, netns, rule, probe]
 
 MAX_ATTEMPTS = 5
 
@@ -157,109 +151,81 @@ class DBProvider(enum.Enum):
     sqlite3 = 'sqlite3'
     psycopg2 = 'psycopg2'
 
-
-class DBConfig:
-    provider = DBProvider.sqlite3
-    spec = ':memory:'
+    def __eq__(self, r):
+        return str(self) == r
 
 
-def publish(method):
-    #
-    # this wrapper will be published in the DBM thread
-    #
-    def _do_local_generator(self, target, request):
-        try:
-            for item in method(self, *request.argv, **request.kwarg):
-                request.response.put(item)
-            request.response.put(StopIteration())
-        except Exception as e:
-            request.response.put(e)
+def publish(f):
+    if isinstance(f, str):
 
-    def _do_local_single(self, target, request):
-        try:
-            (
-                request.response.put(
-                    method(self, *request.argv, **request.kwarg)
-                )
-            )
-        except Exception as e:
-            (request.response.put(e))
+        def decorate(m):
+            m.publish = f
+            return m
 
-    #
-    # this class will be used to map the requests
-    #
-    class cmsg_req(cmsg):
-        def __init__(self, response, *argv, **kwarg):
-            self['header'] = {'target': None}
-            self.response = response
-            self.argv = argv
-            self.kwarg = kwarg
+        return decorate
 
-    #
-    # this method will replace the source one
-    #
-    def _do_dispatch_generator(self, *argv, **kwarg):
-        if self.thread == id(threading.current_thread()):
-            # same thread, run method locally
-            for item in method(self, *argv, **kwarg):
-                yield item
-        else:
-            # another thread, run via message bus
-            self._allow_read.wait()
-            response = queue.Queue()
-            request = cmsg_req(response, *argv, **kwarg)
-            self.event_queue.put((request,))
-            while True:
-                item = response.get()
-                if isinstance(item, StopIteration):
-                    return
-                elif isinstance(item, Exception):
-                    raise item
-                else:
-                    yield item
-
-    def _do_dispatch_single(self, *argv, **kwarg):
-        if self.thread == id(threading.current_thread()):
-            # same thread, run method locally
-            return method(self, *argv, **kwarg)
-        else:
-            # another thread, run via message bus
-            response = queue.Queue(maxsize=1)
-            request = cmsg_req(response, *argv, **kwarg)
-            self.event_queue.put((request,))
-            ret = response.get()
-            if isinstance(ret, Exception):
-                raise ret
-            else:
-                return ret
-
-    #
-    # announce the function so it will be published
-    #
-    if inspect.isgeneratorfunction(method):
-        _do_dispatch_generator.publish = (cmsg_req, _do_local_generator)
-        return _do_dispatch_generator
-
-    _do_dispatch_single.publish = (cmsg_req, _do_local_single)
-    return _do_dispatch_single
+    f.publish = True
+    return f
 
 
-class ReadOnly(object):
-    def __init__(self, schema):
-        self._schema = schema
+class DBDict(dict):
+    def __init__(self, schema, table):
+        self.schema = schema
+        self.table = table
 
-    def __enter__(self):
-        self._schema.allow_write(False)
-        return self
+    @publish('get')
+    def __getitem__(self, key):
+        for (record,) in self.schema.fetch(
+            f'''
+            SELECT f_value FROM {self.table}
+            WHERE f_key = {self.schema.plch}
+            ''',
+            (key,),
+        ):
+            return json.loads(record)
+        raise KeyError(f'key {key} not found')
 
-    def __exit__(self, exc_type, exc_value, traceback):
-        self._schema.allow_write(True)
+    @publish('set')
+    def __setitem__(self, key, value):
+        del self[key]
+        self.schema.execute(
+            f'''
+            INSERT INTO {self.table}
+            VALUES ({self.schema.plch}, {self.schema.plch})
+            ''',
+            (key, json.dumps(value)),
+        )
+
+    @publish('del')
+    def __delitem__(self, key):
+        self.schema.execute(
+            f'''
+            DELETE FROM {self.table}
+            WHERE f_key = {self.schema.plch}
+            ''',
+            (key,),
+        )
+
+    @publish
+    def keys(self):
+        for (key,) in self.schema.fetch(f'SELECT f_key FROM {self.table}'):
+            yield key
+
+    @publish
+    def items(self):
+        for key, value in self.schema.fetch(
+            f'SELECT f_key, f_value FROM {self.table}'
+        ):
+            yield key, json.loads(value)
+
+    @publish
+    def values(self):
+        for (value,) in self.schema.fetch(f'SELECT f_value FROM {self.table}'):
+            yield json.loads(value)
 
 
 class DBSchema:
-
     connection = None
-    thread = None
     event_map = None
     key_defaults = None
     snapshots = None  # <table_name>: <obj_weakref>
@@ -274,34 +240,24 @@ class DBSchema:
     indices = {}
     foreign_keys = {}
 
-    def __init__(
-        self, config, ndb, event_queue, event_map, rtnl_log, log_channel
-    ):
-        # collect all the dispatched methods and publish them
-        for name in dir(self):
-            obj = getattr(self, name, None)
-            if hasattr(obj, 'publish'):
-                event, fbody = obj.publish
-                event_map[event] = [partial(fbody, self)]
-
+    def __init__(self, config, sources, event_map, log_channel):
         global plugins
-        self.ndb = ndb
-        self.config = config
-        self.event_queue = event_queue
+        self.sources = sources
+        self.config = DBDict(self, 'config')
         self.stats = {}
-        self.thread = id(threading.current_thread())
-        self.readonly = ReadOnly(self)
         self.connection = None
         self.cursor = None
-        self.rtnl_log = rtnl_log
         self.log = log_channel
         self.snapshots = {}
         self.key_defaults = {}
         self.event_map = {}
-        self._allow_read = threading.Event()
-        self._allow_read.set()
-        self._allow_write = threading.Event()
-        self._allow_write.set()
+        # cache locally these variables so they will not be
+        # loaded from SQL for every incoming message; this
+        # means also that these variables can not be changed
+        # in runtime
+        self.rtnl_log = config['rtnl_debug']
+        self.provider = config['provider']
+        #
         for plugin in plugins:
             #
             # 1. spec
@@ -316,7 +272,7 @@ class DBSchema:
             for name, cls in plugin.init['classes']:
                 self.classes[name] = cls
         #
-        self.initdb()
+        self.initdb(config)
         #
         for plugin in plugins:
             #
@@ -333,15 +289,15 @@ class DBSchema:
 
         self.gctime = self.ctime = time.time()
 
-    def initdb(self):
+    def initdb(self, config):
         if self.connection is not None:
             self.close()
-        if self.config.provider == DBProvider.sqlite3:
-            self.connection = sqlite3.connect(self.config.spec)
+        if config['provider'] == DBProvider.sqlite3:
+            self.connection = sqlite3.connect(config['spec'])
             self.plch = '?'
             self.connection.execute('PRAGMA foreign_keys = ON')
-        elif self.config.provider == DBProvider.psycopg2:
-            self.connection = psycopg2.connect(**self.config.spec)
+        elif config['provider'] == DBProvider.psycopg2:
+            self.connection = psycopg2.connect(**config['spec'])
             self.plch = '%s'
         else:
             raise TypeError('DB provider not supported')
@@ -368,6 +324,17 @@ class DBSchema:
                      DROP TABLE IF EXISTS sources
                      '''
         )
+        self.execute(
+            '''
+            DROP TABLE IF EXISTS config
+        '''
+        )
+        self.execute(
+            '''
+            CREATE TABLE config
+                (f_key TEXT PRIMARY KEY, f_value TEXT NOT NULL)
+        '''
+        )
         self.execute(
             '''
                      CREATE TABLE IF NOT EXISTS sources
@@ -388,6 +355,8 @@ class DBSchema:
                           ON DELETE CASCADE)
                      '''
         )
+        for key, value in config.items():
+            self.config[key] = value
 
     def merge_spec(self, table1, table2, table, schema_idx):
         spec1 = self.compiled[table1]
@@ -493,6 +462,7 @@ class DBSchema:
             'fset': ','.join(f_set),
             'knames': ','.join(f_idx),
             'fidx': ' AND '.join(f_idx_match),
+            'lookup_fallbacks': iclass.lookup_fallbacks,
         }
 
     @publish
@@ -563,46 +533,12 @@ class DBSchema:
             self.connection.commit()  # no performance optimisation yet
         return self.cursor
 
+    @publish
     def fetchone(self, *argv, **kwarg):
         for row in self.fetch(*argv, **kwarg):
             return row
         return None
 
-    @publish
-    def wait_read(self, timeout=None):
-        return self._allow_read.wait(timeout)
-
-    @publish
-    def wait_write(self, timeout=None):
-        return self._allow_write.wait(timeout)
-
-    def allow_read(self, flag=True):
-        if not flag:
-            # block immediately...
-            self._allow_read.clear()
-        # ...then forward the request through the message bus
-        # in the case of different threads, or simply run stage2
-        self._r_allow_read(flag)
-
-    @publish
-    def _r_allow_read(self, flag):
-        if flag:
-            self._allow_read.set()
-        else:
-            self._allow_read.clear()
-
-    def allow_write(self, flag=True):
-        if not flag:
-            self._allow_write.clear()
-        self._r_allow_write(flag)
-
-    @publish
-    def _r_allow_write(self, flag):
-        if flag:
-            self._allow_write.set()
-        else:
-            self._allow_write.clear()
-
     @publish
     def fetch(self, *argv, **kwarg):
         self.execute(*argv, **kwarg)
@@ -615,10 +551,7 @@ class DBSchema:
 
     @publish
     def backup(self, spec):
-        if (
-            sys.version_info >= (3, 7)
-            and self.config.provider == DBProvider.sqlite3
-        ):
+        if sys.version_info >= (3, 7) and self.provider == DBProvider.sqlite3:
             backup_connection = sqlite3.connect(spec)
             self.connection.backup(backup_connection)
             backup_connection.close()
@@ -649,7 +582,7 @@ class DBSchema:
                 f.close()
 
     def close(self):
-        if self.config.spec != ':memory:':
+        if self.config['spec'] != ':memory:':
             # simply discard in-memory sqlite db on exit
             self.purge_snapshots()
             self.connection.commit()
@@ -846,9 +779,9 @@ class DBSchema:
         for table in tuple(self.snapshots):
             for _ in range(MAX_ATTEMPTS):
                 try:
-                    if self.config.provider == DBProvider.sqlite3:
+                    if self.provider == DBProvider.sqlite3:
                         self.execute('DROP TABLE %s' % table)
-                    elif self.config.provider == DBProvider.psycopg2:
+                    elif self.provider == DBProvider.psycopg2:
                         self.execute('DROP TABLE %s CASCADE' % table)
                     self.connection.commit()
                     del self.snapshots[table]
@@ -974,6 +907,36 @@ class DBSchema:
             compiled = self.compiled[table]
             # a map of sub-NLAs
             nodes = {}
+            # replace
+            r_conditions = []
+            r_values = []
+
+            # Check route replace
+            if (
+                event['header'].get('flags', 0) == NLM_F_REPLACE
+                and event['event'] == 'RTM_NEWROUTE'
+            ):
+                # Replace existing route
+                r_conditions = [table + '.f_target = %s' % self.plch]
+                r_values = [target]
+                for key in self.indices[table]:
+                    if key not in [
+                        'RTA_DST',
+                        'dst_len',
+                        'table',
+                        'RTA_PRIORITY',
+                    ]:
+                        continue
+
+                    r_conditions.append(
+                        table + '.f_%s = %s' % (key, self.plch)
+                    )
+                    value = event.get(key) or event.get_attr(key)
+                    if value is None:
+                        value = self.key_defaults[table][key]
+                    if isinstance(value, (dict, list, tuple, set)):
+                        value = json.dumps(value)
+                    r_values.append(value)
 
             # fetch values (exc. the first two columns)
             for fname, ftype in self.spec[table].items():
@@ -1010,7 +973,13 @@ class DBSchema:
                 values.append(value)
 
             try:
-                if self.config.provider == DBProvider.psycopg2:
+                w_fidx = compiled['fidx']
+                w_ivalues = ivalues
+                if r_conditions:
+                    w_fidx = ' AND '.join(r_conditions)
+                    w_ivalues = r_values
+
+                if self.provider == DBProvider.psycopg2:
                     #
                     # run UPSERT -- the DB provider must support it
                     #
@@ -1025,13 +994,13 @@ class DBSchema:
                                 compiled['plchs'],
                                 compiled['knames'],
                                 compiled['fset'],
-                                compiled['fidx'],
+                                w_fidx,
                             ),
-                            (values + values + ivalues),
+                            (values + values + w_ivalues),
                         )
                     )
                     #
-                elif self.config.provider == DBProvider.sqlite3:
+                elif self.provider == DBProvider.sqlite3:
                     #
                     # SQLite3 >= 3.24 actually has UPSERT, but ...
                     #
@@ -1044,8 +1013,8 @@ class DBSchema:
                             '''
                                       SELECT count(*) FROM %s WHERE %s
                                       '''
-                            % (table, compiled['fidx']),
-                            ivalues,
+                            % (table, w_fidx),
+                            w_ivalues,
                         ).fetchone()
                     )[0]
                     if count == 0:
@@ -1061,8 +1030,8 @@ class DBSchema:
                             '''
                                      UPDATE %s SET %s WHERE %s
                                      '''
-                            % (table, compiled['fset'], compiled['fidx']),
-                            (values + ivalues),
+                            % (table, compiled['fset'], w_fidx),
+                            (values + w_ivalues),
                         )
                 else:
                     raise NotImplementedError()
