@@ -66,6 +66,7 @@ from .objects.address import Address
 from .objects.interface import Interface, Vlan
 from .objects.neighbour import FDBRecord, Neighbour
 from .objects.netns import NetNS
+from .objects.probe import Probe
 from .objects.route import Route
 from .objects.rule import Rule
 from .report import Record, RecordSet
@@ -79,13 +80,13 @@ class TmpHandler:
         self.handler = handler
 
     def __enter__(self):
-        self.ndb.register_handler(
+        self.ndb.task_manager.register_handler(
             self.ndb.schema.classes[self.event], self.handler
         )
         return self
 
     def __exit__(self, exc_type, exc_value, traceback):
-        self.ndb.unregister_handler(
+        self.ndb.task_manager.unregister_handler(
             self.ndb.schema.classes[self.event], self.handler
         )
 
@@ -120,6 +121,7 @@ class View(dict):
         self.classes['routes'] = Route
         self.classes['rules'] = Rule
         self.classes['netns'] = NetNS
+        self.classes['probes'] = Probe
         self.classes['af_bridge_vlans'] = Vlan
 
     def __enter__(self):
@@ -143,7 +145,7 @@ class View(dict):
             return {}
 
     def getmany(self, spec, table=None):
-        return self.ndb.schema.get(table or self.table, spec)
+        return self.ndb.task_manager.db_get(table or self.table, spec)
 
     def getone(self, spec, table=None):
         for obj in self.getmany(spec, table):
@@ -189,6 +191,17 @@ class View(dict):
         spec['create'] = True
         return self[spec]
 
+    @cli.change_pointer
+    @check_auth('obj:modify')
+    def ensure(self, *argspec, **kwspec):
+        try:
+            obj = self.locate(**kwspec)
+        except KeyError:
+            obj = self.create(**kwspec)
+        for key, value in kwspec.items():
+            obj[key] = value
+        return obj
+
     @cli.change_pointer
     @check_auth('obj:modify')
     def add(self, *argspec, **kwspec):
@@ -266,21 +279,26 @@ class View(dict):
         iclass = self.classes[table]
         spec = iclass.new_spec(spec)
         kspec = self.ndb.schema.compiled[table]['norm_idx']
+        lookup_fallbacks = self.ndb.schema.compiled[table]['lookup_fallbacks']
         request = {}
         for name in kspec:
             name = iclass.nla2name(name)
             if name in spec:
                 request[name] = spec[name]
+            elif name in lookup_fallbacks:
+                fallback = lookup_fallbacks[name]
+                if fallback in spec:
+                    request[fallback] = spec[fallback]
+
         if not request:
             raise KeyError('got an empty key')
         return self[request]
 
     @check_auth('obj:read')
     def __getitem__(self, key, table=None):
-
         ret = self.template(key, table)
 
-        # rtnl_object.key() returns a dcitionary that can not
+        # rtnl_object.key() returns a dictionary that can not
         # be used as a cache key. Create here a tuple from it.
         # The key order guaranteed by the dictionary.
         cache_key = tuple(ret.key.items())
@@ -351,6 +369,7 @@ class View(dict):
 
         table = table or self.table
         schema = self.ndb.schema
+        task_manager = self.ndb.task_manager
         names = schema.compiled[self.table]['all_names']
 
         self.log.debug('check if the key %s exists in table %s' % (key, table))
@@ -365,7 +384,7 @@ class View(dict):
                 if isinstance(value, (dict, list, tuple, set)):
                     value = json.dumps(value)
                 values.append(value)
-        spec = schema.fetchone(
+        spec = task_manager.db_fetchone(
             'SELECT * FROM %s WHERE %s' % (self.table, ' AND '.join(keys)),
             values,
         )
@@ -424,13 +443,27 @@ class View(dict):
     @check_auth('obj:list')
     def dump(self):
         iclass = self.classes[self.table]
-        return RecordSet(self._native(iclass.dump(self)))
+        return RecordSet(
+            self._native(iclass.dump(self)),
+            config={
+                'recordset_pipe': self.ndb.config.get(
+                    'recordset_pipe', 'false'
+                )
+            },
+        )
 
     @cli.show_result
     @check_auth('obj:list')
     def summary(self):
         iclass = self.classes[self.table]
-        return RecordSet(self._native(iclass.summary(self)))
+        return RecordSet(
+            self._native(iclass.summary(self)),
+            config={
+                'recordset_pipe': self.ndb.config.get(
+                    'recordset_pipe', 'false'
+                )
+            },
+        )
 
     def __repr__(self):
         if self.chain and 'ifname' in self.chain:
