@@ -204,6 +204,9 @@ You can also make your own NLA descriptions::
 Custom NLA descriptions should be defined in the same class,
 where they are used.
 
+explicit NLA type ids
+~~~~~~~~~~~~~~~~~~~~~
+
 Also, it is possible to use not autogenerated type numbers, as
 for ifaddrmsg, but specify them explicitly::
 
@@ -220,6 +223,87 @@ It is not permitted to mix these two forms in one class: you should
 use ether autogenerated type numbers (two fields tuples), or
 explicit numbers (three fields typles).
 
+nla map adapters
+~~~~~~~~~~~~~~~~
+
+If the default declarative NLA map is not flexible enough, one
+can use a custom map adapter. In order to do so, one should define
+at least one function to return `pyroute2.netlink.NlaSpec()`, and
+one optional function to tell the parser if the attribute is supported.
+The simplest definition only to decode packets:
+
+.. code-block:: python
+
+    from pyroute2.netlink import NlaMapAdapter, NlaSpec, nlmsg
+
+
+    def my_flexible_nla_spec(key):
+        return NlaSpec(nlmsg_atoms.hex, key, f'NLA_CLASS_{key}')
+
+
+    class my_msg(nlmsg):
+
+        nla_map = NlaMapAdapter(my_flexible_nla_spec)
+
+    # example result
+    [
+        {
+            'attrs': [
+                ('NLA_CLASS_1', '00:00:00:00'),
+                ('NLA_CLASS_5', '00:00:00:00'),
+            ],
+            'header': {
+                ...
+            },
+        },
+    ]
+
+In this example the same routine is used both for decoding and encoding
+workflows, but the workflows are not equal, thus the example will fail on
+encoding. Still the example may be useful if you don't plan to encode
+packets of this type.
+
+The decoding workflow will pass an integer as the `key` for NLA type, while
+the encoding workflow passes a string as the `key` for NLA name. To correctly
+handle both workflows, you can use either the `key` type discrimination, or
+the explicit declaration syntax:
+
+.. code-block:: python
+
+    # discriminate workflows by the key type
+    def my_flexible_nla_spec(key):
+        if isinstance(key, int):
+            # decoding workflow
+            ...
+        else:
+            # encoding workflow
+            ...
+
+
+    class my_msg(nlmsg):
+
+        nla_map = NlaMapAdapter(my_flexible_nla_spec)
+
+.. code-block:: python
+
+    # declarate separate workflows
+    def my_flexible_nla_spec_encode(key):
+        # receives a string -- nla type name
+        ...
+
+
+    def my_flexible_nla_spec_decode(key):
+        # receives an int -- nla type id
+        ...
+
+
+    class my_msg(nlmsg):
+
+        nla_map = {
+            'decode': NlaMapAdapter(my_flexible_nla_spec_decode),
+            'encode': NlaMapAdapter(my_flexible_nla_spec_encode),
+        }
+
 array types
 ~~~~~~~~~~~
 
@@ -389,12 +473,12 @@ Please notice, that NLA list *MUST* be mutable.
 
 import io
 import logging
+import socket
 import struct
 import sys
 import threading
 import traceback
 import types
-import typing
 import weakref
 from collections import OrderedDict
 from socket import AF_INET, AF_INET6, AF_UNSPEC, inet_ntop, inet_pton
@@ -453,6 +537,7 @@ CTRL_CMD_GETOPS = 0x6
 CTRL_CMD_NEWMCAST_GRP = 0x7
 CTRL_CMD_DELMCAST_GRP = 0x8
 CTRL_CMD_GETMCAST_GRP = 0x9  # unused
+CTRL_CMD_GETPOLICY = 0xA
 
 
 CTRL_ATTR_UNSPEC = 0x0
@@ -463,6 +548,9 @@ CTRL_ATTR_HDRSIZE = 0x4
 CTRL_ATTR_MAXATTR = 0x5
 CTRL_ATTR_OPS = 0x6
 CTRL_ATTR_MCAST_GROUPS = 0x7
+CTRL_ATTR_POLICY = 0x8
+CTRL_ATTR_OP_POLICY = 0x9
+CTRL_ATTR_OP = 0xA
 
 CTRL_ATTR_OP_UNSPEC = 0x0
 CTRL_ATTR_OP_ID = 0x1
@@ -472,6 +560,36 @@ CTRL_ATTR_MCAST_GRP_UNSPEC = 0x0
 CTRL_ATTR_MCAST_GRP_NAME = 0x1
 CTRL_ATTR_MCAST_GRP_ID = 0x2
 
+NL_ATTR_TYPE_INVALID = 0
+NL_ATTR_TYPE_FLAG = 1
+NL_ATTR_TYPE_U8 = 2
+NL_ATTR_TYPE_U16 = 3
+NL_ATTR_TYPE_U32 = 4
+NL_ATTR_TYPE_U64 = 5
+NL_ATTR_TYPE_S8 = 6
+NL_ATTR_TYPE_S16 = 7
+NL_ATTR_TYPE_S32 = 8
+NL_ATTR_TYPE_S64 = 9
+NL_ATTR_TYPE_BINARY = 10
+NL_ATTR_TYPE_STRING = 11
+NL_ATTR_TYPE_NUL_STRING = 12
+NL_ATTR_TYPE_NESTED = 13
+NL_ATTR_TYPE_NESTED_ARRAY = 14
+NL_ATTR_TYPE_BITFIELD32 = 15
+
+NL_POLICY_TYPE_ATTR_UNSPEC = 0
+NL_POLICY_TYPE_ATTR_TYPE = 1
+NL_POLICY_TYPE_ATTR_MIN_VALUE_S = 2
+NL_POLICY_TYPE_ATTR_MAX_VALUE_S = 3
+NL_POLICY_TYPE_ATTR_MIN_VALUE_U = 4
+NL_POLICY_TYPE_ATTR_MAX_VALUE_U = 5
+NL_POLICY_TYPE_ATTR_MIN_LENGTH = 6
+NL_POLICY_TYPE_ATTR_MAX_LENGTH = 7
+NL_POLICY_TYPE_ATTR_POLICY_IDX = 8
+NL_POLICY_TYPE_ATTR_POLICY_MAXTYPE = 9
+NL_POLICY_TYPE_ATTR_BITFIELD32_MASK = 10
+NL_POLICY_TYPE_ATTR_PAD = 11
+NL_POLICY_TYPE_ATTR_MASK = 12
 
 #  Different Netlink families
 #
@@ -580,7 +698,45 @@ cache_hdr = {}
 cache_jit = {}
 
 
-class SQLSchema(object):
+class NlaSpec(dict):
+    def __init__(
+        self,
+        nla_class,
+        nla_type,
+        nla_name,
+        nla_flags=0,
+        nla_array=False,
+        init=None,
+    ):
+        self.update(
+            {
+                'class': nla_class,
+                'type': nla_type,
+                'name': nla_name,
+                'nla_flags': nla_flags,
+                'nla_array': nla_array,
+                'init': init,
+            }
+        )
+
+
+class NlaMapAdapter:
+    def __init__(self, api_get, api_contains=lambda x: True):
+        self.api_get = api_get
+        self.api_contains = api_contains
+        self.types = None
+
+    def __contains__(self, key):
+        return self.api_contains(key)
+
+    def __getitem__(self, key):
+        ret = self.api_get(key)
+        if isinstance(ret['class'], str):
+            ret['class'] = getattr(self.types, ret['class'])
+        return ret
+
+
+class SQLSchema:
     def __init__(self, cls):
         ret = []
         for field in cls.fields:
@@ -608,12 +764,12 @@ class SQLSchema(object):
                 )
                 ret.append(((nla_name,), sql_type))
 
-        for (fname, ftype) in cls.sql_extra_fields:
+        for fname, ftype in cls.sql_extra_fields:
             if isinstance(fname, basestring):
                 fname = (fname,)
             ret.append((fname, ftype))
 
-        for (dcls, prefix) in cls.sql_extend:
+        for dcls, prefix in cls.sql_extend:
             for fname, ftype in dcls.sql_schema():
                 ret.append(((prefix,) + fname, ftype))
 
@@ -679,6 +835,7 @@ class nlmsg_base(dict):
     sql_constraints = {}
     sql_extra_fields = ()
     sql_extend = ()
+    lookup_fallbacks = {}
     nla_flags = 0  # NLA flags
     value_map = {}
     is_nla = False
@@ -735,11 +892,10 @@ class nlmsg_base(dict):
         self._nla_array = False
         self._nla_flags = self.nla_flags
         self['attrs'] = []
-        self['value'] = NotInitialized
         self.value = NotInitialized
         # work only on non-empty mappings
         if self.nla_map and not self.__class__.__compiled_nla:
-            self.compile_nla()
+            self.compile_nla_table()
         if self.header:
             self['header'] = {}
 
@@ -823,10 +979,18 @@ class nlmsg_base(dict):
         return self
 
     def __ops(self, rvalue, op0, op1):
+        if rvalue is None:
+            return None
         lvalue = self.getvalue()
         res = self.__class__()
+        for key, _ in res.fields:
+            del res[key]
+        if 'header' in res:
+            del res['header']
+        if 'value' in res:
+            del res['value']
         for key in lvalue:
-            if key not in ('header', 'attrs'):
+            if key not in ('header', 'attrs', '__align'):
                 if op0 == '__sub__':
                     # operator -, complement
                     if (key not in rvalue) or (lvalue[key] != rvalue[key]):
@@ -838,11 +1002,13 @@ class nlmsg_base(dict):
         if 'attrs' in lvalue:
             res['attrs'] = []
             for attr in lvalue['attrs']:
-                if isinstance(attr[1], nla):
+                if isinstance(attr[1], nlmsg_base):
+                    print("recursion")
                     diff = getattr(attr[1], op0)(rvalue.get_attr(attr[0]))
                     if diff is not None:
                         res['attrs'].append([attr[0], diff])
                 else:
+                    print("fail", type(attr[1]))
                     if op0 == '__sub__':
                         # operator -, complement
                         if rvalue.get_attr(attr[0]) != attr[1]:
@@ -851,16 +1017,15 @@ class nlmsg_base(dict):
                         # operator &, intersection
                         if rvalue.get_attr(attr[0]) == attr[1]:
                             res['attrs'].append(attr)
+        if 'attrs' in res and not res['attrs']:
+            del res['attrs']
         if not res:
             return None
-        else:
-            if 'header' in res:
-                del res['header']
-            if 'value' in res:
-                del res['value']
-            if 'attrs' in res and not res['attrs']:
-                del res['attrs']
-            return res
+        print(res)
+        return res
+
+    def __bool__(self):
+        return len(self.keys()) > 0
 
     def __sub__(self, rvalue):
         '''
@@ -886,38 +1051,12 @@ class nlmsg_base(dict):
         '''
         lvalue = self.getvalue()
         if lvalue is self:
-            for key in self:
-                try:
-                    if key == 'attrs':
-                        for nla_tuple in self[key]:
-                            lv = self.get_attr(nla_tuple[0])
-                            if isinstance(lv, dict):
-                                lv = nlmsg().setvalue(lv)
-                            rv = rvalue.get_attr(nla_tuple[0])
-                            if isinstance(rv, dict):
-                                rv = nlmsg().setvalue(rv)
-                            # this strange condition means a simple thing:
-                            # None, 0, empty container and NotInitialized in
-                            # that context should be treated as equal.
-                            if (lv != rv) and not (
-                                (not lv or lv is NotInitialized)
-                                and (not rv or rv is NotInitialized)
-                            ):
-                                return False
-                    else:
-                        lv = self.get(key)
-                        rv = rvalue.get(key)
-                        if (lv != rv) and not (
-                            (not lv or lv is NotInitialized)
-                            and (not rv or rv is NotInitialized)
-                        ):
-                            return False
-                except Exception:
-                    # on any error -- is not equal
-                    return False
-            return True
-        else:
-            return lvalue == rvalue
+            if isinstance(rvalue, type(self)):
+                return (self - rvalue) is None
+            if isinstance(rvalue, dict):
+                return dict(self) == rvalue
+            return False
+        return lvalue == rvalue
 
     @classmethod
     def get_size(self):
@@ -1134,12 +1273,9 @@ class nlmsg_base(dict):
         '''
         Universal get() for a netlink message.
         '''
-        if isinstance(key, str) and key in self:
-            ret = self[key]
-        else:
-            if not isinstance(key, typing.Iterable):
-                key = (key,)
-            ret = self.get_nested(*key)
+        if isinstance(key, str):
+            key = (key,)
+        ret = self.get_nested(*key)
         return ret if ret is not None else default
 
     def get_nested(self, *keys):
@@ -1212,8 +1348,15 @@ class nlmsg_base(dict):
     def __getitem__(self, key):
         if isinstance(key, int):
             return self.chain[key]
+        if key == 'value' and key not in self:
+            return NotInitialized
         return dict.__getitem__(self, key)
 
+    def __delitem__(self, key):
+        if key == 'value' and key not in self:
+            return
+        return dict.__delitem__(self, key)
+
     def __setstate__(self, state):
         return self.load(state)
 
@@ -1236,7 +1379,7 @@ class nlmsg_base(dict):
         and `__setstate__()`.
         '''
         if isinstance(dump, dict):
-            for (k, v) in dump.items():
+            for k, v in dump.items():
                 if k == 'header':
                     self['header'].update(dump['header'])
                 else:
@@ -1245,6 +1388,30 @@ class nlmsg_base(dict):
             self.setvalue(dump)
         return self
 
+    def dump_attrs(self, attrs):
+        ret = []
+        for i in attrs:
+            if isinstance(i, nlmsg_base):
+                ret.append(i.dump())
+            elif isinstance(i, (set, list, tuple, nla_slot)):
+                if isinstance(i[1], nlmsg_base):
+                    # catch nlmsg
+                    ret.append([i[0], i[1].dump()])
+                elif isinstance(i[1], (set, list, tuple)):
+                    # catch iterables
+                    ret.append([i[0], self.dump_attrs(i[1])])
+                elif isinstance(i[1], bytes):
+                    ret.append([i[0], hexdump(i[1])])
+                else:
+                    ret.append([i[0], i[1]])
+            elif isinstance(i, dict):
+                if 'attrs' in i:
+                    i['attrs'] = self.dump_attrs(i['attrs'])
+                ret.append(i)
+            else:
+                ret.append(i)
+        return ret
+
     def dump(self):
         '''
         Dump packet as a dict
@@ -1252,20 +1419,15 @@ class nlmsg_base(dict):
         a = self.getvalue()
         if isinstance(a, dict):
             ret = {}
-            for (k, v) in a.items():
+            for k, v in a.items():
                 if k == 'header':
                     ret['header'] = dict(a['header'])
                 elif k == 'attrs':
-                    ret['attrs'] = attrs = []
-                    for i in a['attrs']:
-                        if isinstance(i[1], nlmsg_base):
-                            attrs.append([i[0], i[1].dump()])
-                        elif isinstance(i[1], set):
-                            attrs.append([i[0], tuple(i[1])])
-                        else:
-                            attrs.append([i[0], i[1]])
+                    ret['attrs'] = self.dump_attrs(v)
                 else:
                     ret[k] = v
+        elif isinstance(a, nlmsg_base):
+            ret = a.dump()
         else:
             ret = a
         return ret
@@ -1292,7 +1454,25 @@ class nlmsg_base(dict):
 
         return self
 
-    def compile_nla(self):
+    def compile_nla_table(self):
+        # Bug-Url: https://github.com/svinota/pyroute2/issues/980
+        # Bug-Url: https://github.com/svinota/pyroute2/pull/981
+        if isinstance(self.nla_map, NlaMapAdapter):
+            self.nla_map.types = self
+            self.__class__.__t_nla_map = self.nla_map
+            self.__class__.__r_nla_map = self.nla_map
+            self.__class__.__compiled_nla = True
+            return
+        elif isinstance(self.nla_map, dict):
+            if isinstance(self.nla_map['decode'], NlaMapAdapter):
+                self.nla_map['decode'].types = self
+            if isinstance(self.nla_map['encode'], NlaMapAdapter):
+                self.nla_map['encode'].types = self
+            self.__class__.__t_nla_map = self.nla_map['decode']
+            self.__class__.__r_nla_map = self.nla_map['encode']
+            self.__class__.__compiled_nla = True
+            return
+
         # clean up NLA mappings
         t_nla_map = {}
         r_nla_map = {}
@@ -1319,7 +1499,7 @@ class nlmsg_base(dict):
         else:
             zipped = nla_map
 
-        for (key, name, nla_class, nla_flags) in zipped:
+        for key, name, nla_class, nla_flags in zipped:
             # it is an array
             if nla_class[0] == '*':
                 nla_class = nla_class[1:]
@@ -1337,6 +1517,9 @@ class nlmsg_base(dict):
             # lookup NLA class
             if nla_class == 'recursive':
                 nla_class = type(self)
+            elif nla_class == 'nested':
+                nla_class = type(self)
+                nla_flags |= NLA_F_NESTED
             else:
                 nla_class = getattr(self, nla_class)
             # update mappings
@@ -1416,7 +1599,6 @@ class nlmsg_base(dict):
             length = min(max(length, 4), (self.length - offset + self.offset))
             # we have a mapping for this NLA
             if msg_type in t_nla_map:
-
                 prime = t_nla_map[msg_type]
                 # get the class
                 msg_class = t_nla_map[msg_type]['class']
@@ -1577,7 +1759,6 @@ class nlmsg_encoder_generic(object):
 
 
 class nla_slot(object):
-
     __slots__ = ("cell",)
 
     def __init__(self, name, value):
@@ -1720,119 +1901,102 @@ class nlmsg_atoms(object):
             self.value = True
 
     class uint8(nla_base):
-
         __slots__ = ()
         sql_type = 'INTEGER'
 
         fields = [('value', 'B')]
 
     class uint16(nla_base):
-
         __slots__ = ()
         sql_type = 'INTEGER'
 
         fields = [('value', 'H')]
 
     class uint32(nla_base):
-
         __slots__ = ()
         sql_type = 'BIGINT'
 
         fields = [('value', 'I')]
 
     class uint64(nla_base):
-
         __slots__ = ()
         sql_type = 'BIGINT'
 
         fields = [('value', 'Q')]
 
     class int8(nla_base):
-
         __slots__ = ()
         sql_type = 'INTEGER'
 
         fields = [('value', 'b')]
 
     class int16(nla_base):
-
         __slots__ = ()
         sql_type = 'INTEGER'
 
         fields = [('value', 'h')]
 
     class int32(nla_base):
-
         __slots__ = ()
         sql_type = 'BIGINT'
 
         fields = [('value', 'i')]
 
     class int64(nla_base):
-
         __slots__ = ()
         sql_type = 'BIGINT'
 
         fields = [('value', 'q')]
 
     class be8(nla_base):
-
         __slots__ = ()
         sql_type = 'INTEGER'
 
         fields = [('value', '>B')]
 
     class be16(nla_base):
-
         __slots__ = ()
         sql_type = 'INTEGER'
 
         fields = [('value', '>H')]
 
     class be32(nla_base):
-
         __slots__ = ()
         sql_type = 'BIGINT'
 
         fields = [('value', '>I')]
 
     class be64(nla_base):
-
         __slots__ = ()
         sql_type = 'BIGINT'
 
         fields = [('value', '>Q')]
 
     class sbe8(nla_base):
-
         __slots__ = ()
         sql_type = 'INTEGER'
 
         fields = [('value', '>b')]
 
     class sbe16(nla_base):
-
         __slots__ = ()
         sql_type = 'INTEGER'
 
         fields = [('value', '>h')]
 
     class sbe32(nla_base):
-
         __slots__ = ()
         sql_type = 'BIGINT'
 
         fields = [('value', '>i')]
 
     class sbe64(nla_base):
-
         __slots__ = ()
         sql_type = 'BIGINT'
 
         fields = [('value', '>q')]
 
     class ipXaddr(nla_base_string):
-
         __slots__ = ()
         sql_type = 'TEXT'
 
@@ -1907,15 +2071,37 @@ class nlmsg_atoms(object):
         __slots__ = ()
         sql_type = 'TEXT'
         family = None
+        family_attr = None
         own_parent = True
 
+        def __init__(self, *argv, **kwarg):
+            init = kwarg.get('init', None)
+            if init is not None:
+                key, value = init.split(',')
+                if key == 'family' and value.startswith('AF_'):
+                    self.family = getattr(socket, value)
+                elif key == 'nla':
+                    self.family_attr = value
+            super().__init__(*argv, **kwarg)
+
         def get_family(self):
             if self.family is not None:
                 return self.family
             pointer = self
+            if self.family_attr is not None:
+                nla = self.family_attr
+            else:
+                nla = 'family'
             while pointer.parent is not None:
                 pointer = pointer.parent
-            return pointer.get('family', AF_UNSPEC)
+                family = pointer.get(nla)
+                if family is not None:
+                    return family
+            return AF_UNSPEC
+
+        @staticmethod
+        def get_addrlen(family):
+            return {AF_INET: 4, AF_INET6: 16, AF_MPLS: 4}.get(family, 4)
 
         def encode(self):
             family = self.get_family()
@@ -1952,14 +2138,17 @@ class nlmsg_atoms(object):
         def decode(self):
             nla_base_string.decode(self)
             family = self.get_family()
+            data = self['value']
             if family in (AF_INET, AF_INET6):
-                self.value = inet_ntop(family, self['value'])
+                if family == AF_INET:
+                    data = data[:4]
+                elif family == AF_INET6:
+                    data = data[:16]
+                self.value = inet_ntop(family, data)
             elif family == AF_MPLS:
                 self.value = []
-                for i in range(len(self['value']) // 4):
-                    label = struct.unpack(
-                        '>I', self['value'][i * 4 : i * 4 + 4]
-                    )[0]
+                for i in range(len(data) // 4):
+                    label = struct.unpack('>I', data[i * 4 : i * 4 + 4])[0]
                     record = {
                         'label': (label & 0xFFFFF000) >> 12,
                         'tc': (label & 0x00000E00) >> 9,
@@ -1971,7 +2160,6 @@ class nlmsg_atoms(object):
                 raise TypeError('socket family not supported')
 
     class mpls_target(target):
-
         __slots__ = ()
 
         family = AF_MPLS
@@ -2255,10 +2443,12 @@ class ctrlmsg(genlmsg):
         ('CTRL_ATTR_MAXATTR', 'uint32'),
         ('CTRL_ATTR_OPS', '*ops'),
         ('CTRL_ATTR_MCAST_GROUPS', '*mcast_groups'),
+        ('CTRL_ATTR_POLICY', 'policy_nest'),
+        ('CTRL_ATTR_OP_POLICY', 'command_nest'),
+        ('CTRL_ATTR_OP', 'uint32'),
     )
 
     class ops(nla):
-
         __slots__ = ()
 
         nla_map = (
@@ -2268,7 +2458,6 @@ class ctrlmsg(genlmsg):
         )
 
     class mcast_groups(nla):
-
         __slots__ = ()
 
         nla_map = (
@@ -2276,3 +2465,67 @@ class ctrlmsg(genlmsg):
             ('CTRL_ATTR_MCAST_GRP_NAME', 'asciiz'),
             ('CTRL_ATTR_MCAST_GRP_ID', 'uint32'),
         )
+
+    class policy_nest(nla):
+        __slots__ = ()
+
+        nla_map = {
+            'decode': NlaMapAdapter(
+                lambda x: NlaSpec('attribute_nest', x, f'POLICY({x})')
+            ),
+            'encode': NlaMapAdapter(
+                lambda x: NlaSpec('attribute_nest', int(x[7:-1]), x)
+            ),
+        }
+
+        class attribute_nest(nla):
+            __slots__ = ()
+
+            nla_map = {
+                'decode': NlaMapAdapter(
+                    lambda x: NlaSpec('nl_policy_type_attr', x, f'ATTR({x})')
+                ),
+                'encode': NlaMapAdapter(
+                    lambda x: NlaSpec('nl_policy_type_attr', int(x[5:-1]), x)
+                ),
+            }
+
+            class nl_policy_type_attr(nla):
+                __slots__ = ()
+
+                nla_map = (
+                    ('NL_POLICY_TYPE_ATTR_UNSPEC', 'none'),
+                    ('NL_POLICY_TYPE_ATTR_TYPE', 'uint32'),
+                    ('NL_POLICY_TYPE_ATTR_MIN_VALUE_S', 'int64'),
+                    ('NL_POLICY_TYPE_ATTR_MAX_VALUE_S', 'int64'),
+                    ('NL_POLICY_TYPE_ATTR_MIN_VALUE_U', 'int64'),
+                    ('NL_POLICY_TYPE_ATTR_MAX_VALUE_U', 'int64'),
+                    ('NL_POLICY_TYPE_ATTR_MIN_LENGTH', 'uint32'),
+                    ('NL_POLICY_TYPE_ATTR_MAX_LENGTH', 'uint32'),
+                    ('NL_POLICY_TYPE_ATTR_POLICY_IDX', 'uint32'),
+                    ('NL_POLICY_TYPE_ATTR_POLICY_MAXTYPE', 'uint32'),
+                    ('NL_POLICY_TYPE_ATTR_BITFIELD32_MASK', 'uint32'),
+                    ('NL_POLICY_TYPE_ATTR_PAD', 'uint64'),
+                    ('NL_POLICY_TYPE_ATTR_MASK', 'uint64'),
+                )
+
+    class command_nest(nla):
+        __slots__ = ()
+
+        nla_map = {
+            'decode': NlaMapAdapter(
+                lambda x: NlaSpec('command_nest_attrs', x, f'OP({x})')
+            ),
+            'encode': NlaMapAdapter(
+                lambda x: NlaSpec('command_nest_attrs', int(x[3:-1]), x)
+            ),
+        }
+
+        class command_nest_attrs(nla):
+            __slots__ = ()
+
+            nla_map = (
+                ('CTRL_ATTR_POLICY_UNSPEC', 'none'),
+                ('CTRL_ATTR_POLICY_DO', 'uint32'),
+                ('CTRL_ATTR_POLICY_DUMP', 'uint32'),
+            )
