@@ -1,7 +1,10 @@
 import pytest
+from pr2test.marks import require_root
 
 from pyroute2 import IPRoute
 
+pytestmark = [require_root()]
+
 
 @pytest.mark.parametrize('nlm_echo', (True, False))
 def test_echo_route(context, nlm_echo):
