@@ -20,6 +20,17 @@ def test_updown_link(context):
     assert not (context.ipr.get_links(ifname=ifname)[0]['flags'] & 1)
 
 
+@skip_if_not_supported
+@pytest.mark.parametrize('context', test_matrix, indirect=True)
+def test_link_altname_lookup(context):
+    altname = context.new_ifname
+    index, ifname = context.default_interface
+    context.ipr.link('property_add', index=index, altname=altname)
+    assert len(context.ipr.link('get', altname=altname)) == 1
+    assert context.ipr.link_lookup(ifname=ifname) == [index]
+    assert context.ipr.link_lookup(altname=altname) == [index]
+
+
 @skip_if_not_supported
 @pytest.mark.parametrize('context', test_matrix, indirect=True)
 def test_link_altname(context):
@@ -125,7 +136,6 @@ def test_remove_link(context):
 
 @pytest.mark.parametrize('context', test_matrix, indirect=True)
 def test_brport_basic(context):
-
     bridge = context.new_ifname
     port = context.new_ifname
 
