@@ -30,7 +30,7 @@ def ctx():
     ndb.close()
 
 
-def test_client_module(ctx):
+def _do_test_client_module(ctx):
     if ctx.index == 0:
         pytest.skip('no DHCP interfaces detected')
 
@@ -46,17 +46,18 @@ def test_client_module(ctx):
         hexdump(hexload(l2addr)) == ctx.ndb.interfaces[ctx.ifname]['address']
     )
     assert router == ctx.ndb.routes['default']['gateway']
-    assert {
-        'address': address,
-        'prefixlen': prefixlen,
-        'index': ctx.index,
-    } in ctx.ndb.addresses
     assert options['lease_time'] > 0
+    assert prefixlen > 0
+    assert address is not None
     return response
 
 
+def test_client_module(ctx):
+    _do_test_client_module(ctx)
+
+
 def test_client_console(ctx):
-    response_from_module = json.loads(json.dumps(test_client_module(ctx)))
+    response_from_module = json.loads(json.dumps(_do_test_client_module(ctx)))
     client = subprocess.run(
         ['pyroute2-dhcp-client', ctx.ifname], stdout=subprocess.PIPE
     )
